# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['ClusterArgs', 'Cluster']

@pulumi.input_type
class ClusterArgs:
    def __init__(__self__, *,
                 compartment_id: pulumi.Input[str],
                 data_node_count: pulumi.Input[int],
                 data_node_host_memory_gb: pulumi.Input[int],
                 data_node_host_ocpu_count: pulumi.Input[int],
                 data_node_host_type: pulumi.Input[str],
                 data_node_storage_gb: pulumi.Input[int],
                 display_name: pulumi.Input[str],
                 master_node_count: pulumi.Input[int],
                 master_node_host_memory_gb: pulumi.Input[int],
                 master_node_host_ocpu_count: pulumi.Input[int],
                 master_node_host_type: pulumi.Input[str],
                 opendashboard_node_count: pulumi.Input[int],
                 opendashboard_node_host_memory_gb: pulumi.Input[int],
                 opendashboard_node_host_ocpu_count: pulumi.Input[int],
                 software_version: pulumi.Input[str],
                 subnet_compartment_id: pulumi.Input[str],
                 subnet_id: pulumi.Input[str],
                 vcn_compartment_id: pulumi.Input[str],
                 vcn_id: pulumi.Input[str],
                 data_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 master_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 security_master_user_name: Optional[pulumi.Input[str]] = None,
                 security_master_user_password_hash: Optional[pulumi.Input[str]] = None,
                 security_mode: Optional[pulumi.Input[str]] = None,
                 system_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None):
        """
        The set of arguments for constructing a Cluster resource.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment to create the cluster in.
        :param pulumi.Input[int] data_node_count: (Updatable) The number of data nodes to configure for the cluster.
        :param pulumi.Input[int] data_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        :param pulumi.Input[str] data_node_host_type: TThe instance type for the cluster's data nodes.
        :param pulumi.Input[int] data_node_storage_gb: (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[str] display_name: (Updatable) The name of the cluster. Avoid entering confidential information.
        :param pulumi.Input[int] master_node_count: (Updatable) The number of master nodes to configure for the cluster.
        :param pulumi.Input[int] master_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        :param pulumi.Input[str] master_node_host_type: The instance type for the cluster's master nodes.
        :param pulumi.Input[int] opendashboard_node_count: (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        :param pulumi.Input[int] opendashboard_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[int] opendashboard_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[str] software_version: (Updatable) The version of the software the cluster is running.
        :param pulumi.Input[str] subnet_compartment_id: The OCID for the compartment where the cluster's subnet is located.
        :param pulumi.Input[str] subnet_id: The OCID of the cluster's subnet.
        :param pulumi.Input[str] vcn_compartment_id: The OCID for the compartment where the cluster's VCN is located.
        :param pulumi.Input[str] vcn_id: The OCID of the cluster's VCN.
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        :param pulumi.Input[str] data_node_host_bare_metal_shape: The bare metal shape for the cluster's data nodes.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] master_node_host_bare_metal_shape: The bare metal shape for the cluster's master nodes.
        :param pulumi.Input[str] security_master_user_name: (Updatable) The name of the master user that are used to manage security config
        :param pulumi.Input[str] security_master_user_password_hash: (Updatable) The password hash of the master user that are used to manage security config
        :param pulumi.Input[str] security_mode: (Updatable) The security mode of the cluster.
        :param pulumi.Input[Mapping[str, Any]] system_tags: Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        """
        pulumi.set(__self__, "compartment_id", compartment_id)
        pulumi.set(__self__, "data_node_count", data_node_count)
        pulumi.set(__self__, "data_node_host_memory_gb", data_node_host_memory_gb)
        pulumi.set(__self__, "data_node_host_ocpu_count", data_node_host_ocpu_count)
        pulumi.set(__self__, "data_node_host_type", data_node_host_type)
        pulumi.set(__self__, "data_node_storage_gb", data_node_storage_gb)
        pulumi.set(__self__, "display_name", display_name)
        pulumi.set(__self__, "master_node_count", master_node_count)
        pulumi.set(__self__, "master_node_host_memory_gb", master_node_host_memory_gb)
        pulumi.set(__self__, "master_node_host_ocpu_count", master_node_host_ocpu_count)
        pulumi.set(__self__, "master_node_host_type", master_node_host_type)
        pulumi.set(__self__, "opendashboard_node_count", opendashboard_node_count)
        pulumi.set(__self__, "opendashboard_node_host_memory_gb", opendashboard_node_host_memory_gb)
        pulumi.set(__self__, "opendashboard_node_host_ocpu_count", opendashboard_node_host_ocpu_count)
        pulumi.set(__self__, "software_version", software_version)
        pulumi.set(__self__, "subnet_compartment_id", subnet_compartment_id)
        pulumi.set(__self__, "subnet_id", subnet_id)
        pulumi.set(__self__, "vcn_compartment_id", vcn_compartment_id)
        pulumi.set(__self__, "vcn_id", vcn_id)
        if data_node_host_bare_metal_shape is not None:
            pulumi.set(__self__, "data_node_host_bare_metal_shape", data_node_host_bare_metal_shape)
        if defined_tags is not None:
            pulumi.set(__self__, "defined_tags", defined_tags)
        if freeform_tags is not None:
            pulumi.set(__self__, "freeform_tags", freeform_tags)
        if master_node_host_bare_metal_shape is not None:
            pulumi.set(__self__, "master_node_host_bare_metal_shape", master_node_host_bare_metal_shape)
        if security_master_user_name is not None:
            pulumi.set(__self__, "security_master_user_name", security_master_user_name)
        if security_master_user_password_hash is not None:
            pulumi.set(__self__, "security_master_user_password_hash", security_master_user_password_hash)
        if security_mode is not None:
            pulumi.set(__self__, "security_mode", security_mode)
        if system_tags is not None:
            pulumi.set(__self__, "system_tags", system_tags)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Input[str]:
        """
        The OCID of the compartment to create the cluster in.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="dataNodeCount")
    def data_node_count(self) -> pulumi.Input[int]:
        """
        (Updatable) The number of data nodes to configure for the cluster.
        """
        return pulumi.get(self, "data_node_count")

    @data_node_count.setter
    def data_node_count(self, value: pulumi.Input[int]):
        pulumi.set(self, "data_node_count", value)

    @property
    @pulumi.getter(name="dataNodeHostMemoryGb")
    def data_node_host_memory_gb(self) -> pulumi.Input[int]:
        """
        (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_memory_gb")

    @data_node_host_memory_gb.setter
    def data_node_host_memory_gb(self, value: pulumi.Input[int]):
        pulumi.set(self, "data_node_host_memory_gb", value)

    @property
    @pulumi.getter(name="dataNodeHostOcpuCount")
    def data_node_host_ocpu_count(self) -> pulumi.Input[int]:
        """
        (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_ocpu_count")

    @data_node_host_ocpu_count.setter
    def data_node_host_ocpu_count(self, value: pulumi.Input[int]):
        pulumi.set(self, "data_node_host_ocpu_count", value)

    @property
    @pulumi.getter(name="dataNodeHostType")
    def data_node_host_type(self) -> pulumi.Input[str]:
        """
        TThe instance type for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_type")

    @data_node_host_type.setter
    def data_node_host_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "data_node_host_type", value)

    @property
    @pulumi.getter(name="dataNodeStorageGb")
    def data_node_storage_gb(self) -> pulumi.Input[int]:
        """
        (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_storage_gb")

    @data_node_storage_gb.setter
    def data_node_storage_gb(self, value: pulumi.Input[int]):
        pulumi.set(self, "data_node_storage_gb", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Input[str]:
        """
        (Updatable) The name of the cluster. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="masterNodeCount")
    def master_node_count(self) -> pulumi.Input[int]:
        """
        (Updatable) The number of master nodes to configure for the cluster.
        """
        return pulumi.get(self, "master_node_count")

    @master_node_count.setter
    def master_node_count(self, value: pulumi.Input[int]):
        pulumi.set(self, "master_node_count", value)

    @property
    @pulumi.getter(name="masterNodeHostMemoryGb")
    def master_node_host_memory_gb(self) -> pulumi.Input[int]:
        """
        (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_memory_gb")

    @master_node_host_memory_gb.setter
    def master_node_host_memory_gb(self, value: pulumi.Input[int]):
        pulumi.set(self, "master_node_host_memory_gb", value)

    @property
    @pulumi.getter(name="masterNodeHostOcpuCount")
    def master_node_host_ocpu_count(self) -> pulumi.Input[int]:
        """
        (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        """
        return pulumi.get(self, "master_node_host_ocpu_count")

    @master_node_host_ocpu_count.setter
    def master_node_host_ocpu_count(self, value: pulumi.Input[int]):
        pulumi.set(self, "master_node_host_ocpu_count", value)

    @property
    @pulumi.getter(name="masterNodeHostType")
    def master_node_host_type(self) -> pulumi.Input[str]:
        """
        The instance type for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_type")

    @master_node_host_type.setter
    def master_node_host_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "master_node_host_type", value)

    @property
    @pulumi.getter(name="opendashboardNodeCount")
    def opendashboard_node_count(self) -> pulumi.Input[int]:
        """
        (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        """
        return pulumi.get(self, "opendashboard_node_count")

    @opendashboard_node_count.setter
    def opendashboard_node_count(self, value: pulumi.Input[int]):
        pulumi.set(self, "opendashboard_node_count", value)

    @property
    @pulumi.getter(name="opendashboardNodeHostMemoryGb")
    def opendashboard_node_host_memory_gb(self) -> pulumi.Input[int]:
        """
        (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        """
        return pulumi.get(self, "opendashboard_node_host_memory_gb")

    @opendashboard_node_host_memory_gb.setter
    def opendashboard_node_host_memory_gb(self, value: pulumi.Input[int]):
        pulumi.set(self, "opendashboard_node_host_memory_gb", value)

    @property
    @pulumi.getter(name="opendashboardNodeHostOcpuCount")
    def opendashboard_node_host_ocpu_count(self) -> pulumi.Input[int]:
        """
        (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        """
        return pulumi.get(self, "opendashboard_node_host_ocpu_count")

    @opendashboard_node_host_ocpu_count.setter
    def opendashboard_node_host_ocpu_count(self, value: pulumi.Input[int]):
        pulumi.set(self, "opendashboard_node_host_ocpu_count", value)

    @property
    @pulumi.getter(name="softwareVersion")
    def software_version(self) -> pulumi.Input[str]:
        """
        (Updatable) The version of the software the cluster is running.
        """
        return pulumi.get(self, "software_version")

    @software_version.setter
    def software_version(self, value: pulumi.Input[str]):
        pulumi.set(self, "software_version", value)

    @property
    @pulumi.getter(name="subnetCompartmentId")
    def subnet_compartment_id(self) -> pulumi.Input[str]:
        """
        The OCID for the compartment where the cluster's subnet is located.
        """
        return pulumi.get(self, "subnet_compartment_id")

    @subnet_compartment_id.setter
    def subnet_compartment_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "subnet_compartment_id", value)

    @property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> pulumi.Input[str]:
        """
        The OCID of the cluster's subnet.
        """
        return pulumi.get(self, "subnet_id")

    @subnet_id.setter
    def subnet_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "subnet_id", value)

    @property
    @pulumi.getter(name="vcnCompartmentId")
    def vcn_compartment_id(self) -> pulumi.Input[str]:
        """
        The OCID for the compartment where the cluster's VCN is located.
        """
        return pulumi.get(self, "vcn_compartment_id")

    @vcn_compartment_id.setter
    def vcn_compartment_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "vcn_compartment_id", value)

    @property
    @pulumi.getter(name="vcnId")
    def vcn_id(self) -> pulumi.Input[str]:
        """
        The OCID of the cluster's VCN.


        ** IMPORTANT **
        Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        return pulumi.get(self, "vcn_id")

    @vcn_id.setter
    def vcn_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "vcn_id", value)

    @property
    @pulumi.getter(name="dataNodeHostBareMetalShape")
    def data_node_host_bare_metal_shape(self) -> Optional[pulumi.Input[str]]:
        """
        The bare metal shape for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_bare_metal_shape")

    @data_node_host_bare_metal_shape.setter
    def data_node_host_bare_metal_shape(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "data_node_host_bare_metal_shape", value)

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @defined_tags.setter
    def defined_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "defined_tags", value)

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @freeform_tags.setter
    def freeform_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "freeform_tags", value)

    @property
    @pulumi.getter(name="masterNodeHostBareMetalShape")
    def master_node_host_bare_metal_shape(self) -> Optional[pulumi.Input[str]]:
        """
        The bare metal shape for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_bare_metal_shape")

    @master_node_host_bare_metal_shape.setter
    def master_node_host_bare_metal_shape(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "master_node_host_bare_metal_shape", value)

    @property
    @pulumi.getter(name="securityMasterUserName")
    def security_master_user_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The name of the master user that are used to manage security config
        """
        return pulumi.get(self, "security_master_user_name")

    @security_master_user_name.setter
    def security_master_user_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_master_user_name", value)

    @property
    @pulumi.getter(name="securityMasterUserPasswordHash")
    def security_master_user_password_hash(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The password hash of the master user that are used to manage security config
        """
        return pulumi.get(self, "security_master_user_password_hash")

    @security_master_user_password_hash.setter
    def security_master_user_password_hash(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_master_user_password_hash", value)

    @property
    @pulumi.getter(name="securityMode")
    def security_mode(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The security mode of the cluster.
        """
        return pulumi.get(self, "security_mode")

    @security_mode.setter
    def security_mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_mode", value)

    @property
    @pulumi.getter(name="systemTags")
    def system_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        """
        return pulumi.get(self, "system_tags")

    @system_tags.setter
    def system_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "system_tags", value)


@pulumi.input_type
class _ClusterState:
    def __init__(__self__, *,
                 availability_domains: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 data_node_count: Optional[pulumi.Input[int]] = None,
                 data_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 data_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 data_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 data_node_host_type: Optional[pulumi.Input[str]] = None,
                 data_node_storage_gb: Optional[pulumi.Input[int]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 fqdn: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 lifecycle_details: Optional[pulumi.Input[str]] = None,
                 master_node_count: Optional[pulumi.Input[int]] = None,
                 master_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 master_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 master_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 master_node_host_type: Optional[pulumi.Input[str]] = None,
                 opendashboard_fqdn: Optional[pulumi.Input[str]] = None,
                 opendashboard_node_count: Optional[pulumi.Input[int]] = None,
                 opendashboard_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 opendashboard_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 opendashboard_private_ip: Optional[pulumi.Input[str]] = None,
                 opensearch_fqdn: Optional[pulumi.Input[str]] = None,
                 opensearch_private_ip: Optional[pulumi.Input[str]] = None,
                 security_master_user_name: Optional[pulumi.Input[str]] = None,
                 security_master_user_password_hash: Optional[pulumi.Input[str]] = None,
                 security_mode: Optional[pulumi.Input[str]] = None,
                 software_version: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 subnet_compartment_id: Optional[pulumi.Input[str]] = None,
                 subnet_id: Optional[pulumi.Input[str]] = None,
                 system_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 time_created: Optional[pulumi.Input[str]] = None,
                 time_deleted: Optional[pulumi.Input[str]] = None,
                 time_updated: Optional[pulumi.Input[str]] = None,
                 total_storage_gb: Optional[pulumi.Input[int]] = None,
                 vcn_compartment_id: Optional[pulumi.Input[str]] = None,
                 vcn_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Cluster resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] availability_domains: The availability domains to distribute the cluser nodes across.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment to create the cluster in.
        :param pulumi.Input[int] data_node_count: (Updatable) The number of data nodes to configure for the cluster.
        :param pulumi.Input[str] data_node_host_bare_metal_shape: The bare metal shape for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        :param pulumi.Input[str] data_node_host_type: TThe instance type for the cluster's data nodes.
        :param pulumi.Input[int] data_node_storage_gb: (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] display_name: (Updatable) The name of the cluster. Avoid entering confidential information.
        :param pulumi.Input[str] fqdn: The fully qualified domain name (FQDN) for the cluster's API endpoint.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] lifecycle_details: Additional information about the current lifecycle state of the cluster.
        :param pulumi.Input[int] master_node_count: (Updatable) The number of master nodes to configure for the cluster.
        :param pulumi.Input[str] master_node_host_bare_metal_shape: The bare metal shape for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        :param pulumi.Input[str] master_node_host_type: The instance type for the cluster's master nodes.
        :param pulumi.Input[str] opendashboard_fqdn: The fully qualified domain name (FQDN) for the cluster's OpenSearch Dashboard API endpoint.
        :param pulumi.Input[int] opendashboard_node_count: (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        :param pulumi.Input[int] opendashboard_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[int] opendashboard_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[str] opendashboard_private_ip: The private IP address for the cluster's OpenSearch Dashboard.
        :param pulumi.Input[str] opensearch_fqdn: The fully qualified domain name (FQDN) for the cluster's API endpoint.
        :param pulumi.Input[str] opensearch_private_ip: The cluster's private IP address.
        :param pulumi.Input[str] security_master_user_name: (Updatable) The name of the master user that are used to manage security config
        :param pulumi.Input[str] security_master_user_password_hash: (Updatable) The password hash of the master user that are used to manage security config
        :param pulumi.Input[str] security_mode: (Updatable) The security mode of the cluster.
        :param pulumi.Input[str] software_version: (Updatable) The version of the software the cluster is running.
        :param pulumi.Input[str] state: The current state of the cluster.
        :param pulumi.Input[str] subnet_compartment_id: The OCID for the compartment where the cluster's subnet is located.
        :param pulumi.Input[str] subnet_id: The OCID of the cluster's subnet.
        :param pulumi.Input[Mapping[str, Any]] system_tags: Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        :param pulumi.Input[str] time_created: The amount of time in milliseconds since the cluster was created.
        :param pulumi.Input[str] time_deleted: The amount of time in milliseconds since the cluster was updated.
        :param pulumi.Input[str] time_updated: The amount of time in milliseconds since the cluster was updated.
        :param pulumi.Input[int] total_storage_gb: The size in GB of the cluster's total storage.
        :param pulumi.Input[str] vcn_compartment_id: The OCID for the compartment where the cluster's VCN is located.
        :param pulumi.Input[str] vcn_id: The OCID of the cluster's VCN.
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        if availability_domains is not None:
            pulumi.set(__self__, "availability_domains", availability_domains)
        if compartment_id is not None:
            pulumi.set(__self__, "compartment_id", compartment_id)
        if data_node_count is not None:
            pulumi.set(__self__, "data_node_count", data_node_count)
        if data_node_host_bare_metal_shape is not None:
            pulumi.set(__self__, "data_node_host_bare_metal_shape", data_node_host_bare_metal_shape)
        if data_node_host_memory_gb is not None:
            pulumi.set(__self__, "data_node_host_memory_gb", data_node_host_memory_gb)
        if data_node_host_ocpu_count is not None:
            pulumi.set(__self__, "data_node_host_ocpu_count", data_node_host_ocpu_count)
        if data_node_host_type is not None:
            pulumi.set(__self__, "data_node_host_type", data_node_host_type)
        if data_node_storage_gb is not None:
            pulumi.set(__self__, "data_node_storage_gb", data_node_storage_gb)
        if defined_tags is not None:
            pulumi.set(__self__, "defined_tags", defined_tags)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if fqdn is not None:
            pulumi.set(__self__, "fqdn", fqdn)
        if freeform_tags is not None:
            pulumi.set(__self__, "freeform_tags", freeform_tags)
        if lifecycle_details is not None:
            pulumi.set(__self__, "lifecycle_details", lifecycle_details)
        if master_node_count is not None:
            pulumi.set(__self__, "master_node_count", master_node_count)
        if master_node_host_bare_metal_shape is not None:
            pulumi.set(__self__, "master_node_host_bare_metal_shape", master_node_host_bare_metal_shape)
        if master_node_host_memory_gb is not None:
            pulumi.set(__self__, "master_node_host_memory_gb", master_node_host_memory_gb)
        if master_node_host_ocpu_count is not None:
            pulumi.set(__self__, "master_node_host_ocpu_count", master_node_host_ocpu_count)
        if master_node_host_type is not None:
            pulumi.set(__self__, "master_node_host_type", master_node_host_type)
        if opendashboard_fqdn is not None:
            pulumi.set(__self__, "opendashboard_fqdn", opendashboard_fqdn)
        if opendashboard_node_count is not None:
            pulumi.set(__self__, "opendashboard_node_count", opendashboard_node_count)
        if opendashboard_node_host_memory_gb is not None:
            pulumi.set(__self__, "opendashboard_node_host_memory_gb", opendashboard_node_host_memory_gb)
        if opendashboard_node_host_ocpu_count is not None:
            pulumi.set(__self__, "opendashboard_node_host_ocpu_count", opendashboard_node_host_ocpu_count)
        if opendashboard_private_ip is not None:
            pulumi.set(__self__, "opendashboard_private_ip", opendashboard_private_ip)
        if opensearch_fqdn is not None:
            pulumi.set(__self__, "opensearch_fqdn", opensearch_fqdn)
        if opensearch_private_ip is not None:
            pulumi.set(__self__, "opensearch_private_ip", opensearch_private_ip)
        if security_master_user_name is not None:
            pulumi.set(__self__, "security_master_user_name", security_master_user_name)
        if security_master_user_password_hash is not None:
            pulumi.set(__self__, "security_master_user_password_hash", security_master_user_password_hash)
        if security_mode is not None:
            pulumi.set(__self__, "security_mode", security_mode)
        if software_version is not None:
            pulumi.set(__self__, "software_version", software_version)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if subnet_compartment_id is not None:
            pulumi.set(__self__, "subnet_compartment_id", subnet_compartment_id)
        if subnet_id is not None:
            pulumi.set(__self__, "subnet_id", subnet_id)
        if system_tags is not None:
            pulumi.set(__self__, "system_tags", system_tags)
        if time_created is not None:
            pulumi.set(__self__, "time_created", time_created)
        if time_deleted is not None:
            pulumi.set(__self__, "time_deleted", time_deleted)
        if time_updated is not None:
            pulumi.set(__self__, "time_updated", time_updated)
        if total_storage_gb is not None:
            pulumi.set(__self__, "total_storage_gb", total_storage_gb)
        if vcn_compartment_id is not None:
            pulumi.set(__self__, "vcn_compartment_id", vcn_compartment_id)
        if vcn_id is not None:
            pulumi.set(__self__, "vcn_id", vcn_id)

    @property
    @pulumi.getter(name="availabilityDomains")
    def availability_domains(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The availability domains to distribute the cluser nodes across.
        """
        return pulumi.get(self, "availability_domains")

    @availability_domains.setter
    def availability_domains(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "availability_domains", value)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID of the compartment to create the cluster in.
        """
        return pulumi.get(self, "compartment_id")

    @compartment_id.setter
    def compartment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "compartment_id", value)

    @property
    @pulumi.getter(name="dataNodeCount")
    def data_node_count(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The number of data nodes to configure for the cluster.
        """
        return pulumi.get(self, "data_node_count")

    @data_node_count.setter
    def data_node_count(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "data_node_count", value)

    @property
    @pulumi.getter(name="dataNodeHostBareMetalShape")
    def data_node_host_bare_metal_shape(self) -> Optional[pulumi.Input[str]]:
        """
        The bare metal shape for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_bare_metal_shape")

    @data_node_host_bare_metal_shape.setter
    def data_node_host_bare_metal_shape(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "data_node_host_bare_metal_shape", value)

    @property
    @pulumi.getter(name="dataNodeHostMemoryGb")
    def data_node_host_memory_gb(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_memory_gb")

    @data_node_host_memory_gb.setter
    def data_node_host_memory_gb(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "data_node_host_memory_gb", value)

    @property
    @pulumi.getter(name="dataNodeHostOcpuCount")
    def data_node_host_ocpu_count(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_ocpu_count")

    @data_node_host_ocpu_count.setter
    def data_node_host_ocpu_count(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "data_node_host_ocpu_count", value)

    @property
    @pulumi.getter(name="dataNodeHostType")
    def data_node_host_type(self) -> Optional[pulumi.Input[str]]:
        """
        TThe instance type for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_type")

    @data_node_host_type.setter
    def data_node_host_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "data_node_host_type", value)

    @property
    @pulumi.getter(name="dataNodeStorageGb")
    def data_node_storage_gb(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_storage_gb")

    @data_node_storage_gb.setter
    def data_node_storage_gb(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "data_node_storage_gb", value)

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @defined_tags.setter
    def defined_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "defined_tags", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The name of the cluster. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter
    def fqdn(self) -> Optional[pulumi.Input[str]]:
        """
        The fully qualified domain name (FQDN) for the cluster's API endpoint.
        """
        return pulumi.get(self, "fqdn")

    @fqdn.setter
    def fqdn(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fqdn", value)

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @freeform_tags.setter
    def freeform_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "freeform_tags", value)

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> Optional[pulumi.Input[str]]:
        """
        Additional information about the current lifecycle state of the cluster.
        """
        return pulumi.get(self, "lifecycle_details")

    @lifecycle_details.setter
    def lifecycle_details(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "lifecycle_details", value)

    @property
    @pulumi.getter(name="masterNodeCount")
    def master_node_count(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The number of master nodes to configure for the cluster.
        """
        return pulumi.get(self, "master_node_count")

    @master_node_count.setter
    def master_node_count(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "master_node_count", value)

    @property
    @pulumi.getter(name="masterNodeHostBareMetalShape")
    def master_node_host_bare_metal_shape(self) -> Optional[pulumi.Input[str]]:
        """
        The bare metal shape for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_bare_metal_shape")

    @master_node_host_bare_metal_shape.setter
    def master_node_host_bare_metal_shape(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "master_node_host_bare_metal_shape", value)

    @property
    @pulumi.getter(name="masterNodeHostMemoryGb")
    def master_node_host_memory_gb(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_memory_gb")

    @master_node_host_memory_gb.setter
    def master_node_host_memory_gb(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "master_node_host_memory_gb", value)

    @property
    @pulumi.getter(name="masterNodeHostOcpuCount")
    def master_node_host_ocpu_count(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        """
        return pulumi.get(self, "master_node_host_ocpu_count")

    @master_node_host_ocpu_count.setter
    def master_node_host_ocpu_count(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "master_node_host_ocpu_count", value)

    @property
    @pulumi.getter(name="masterNodeHostType")
    def master_node_host_type(self) -> Optional[pulumi.Input[str]]:
        """
        The instance type for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_type")

    @master_node_host_type.setter
    def master_node_host_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "master_node_host_type", value)

    @property
    @pulumi.getter(name="opendashboardFqdn")
    def opendashboard_fqdn(self) -> Optional[pulumi.Input[str]]:
        """
        The fully qualified domain name (FQDN) for the cluster's OpenSearch Dashboard API endpoint.
        """
        return pulumi.get(self, "opendashboard_fqdn")

    @opendashboard_fqdn.setter
    def opendashboard_fqdn(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "opendashboard_fqdn", value)

    @property
    @pulumi.getter(name="opendashboardNodeCount")
    def opendashboard_node_count(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        """
        return pulumi.get(self, "opendashboard_node_count")

    @opendashboard_node_count.setter
    def opendashboard_node_count(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "opendashboard_node_count", value)

    @property
    @pulumi.getter(name="opendashboardNodeHostMemoryGb")
    def opendashboard_node_host_memory_gb(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        """
        return pulumi.get(self, "opendashboard_node_host_memory_gb")

    @opendashboard_node_host_memory_gb.setter
    def opendashboard_node_host_memory_gb(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "opendashboard_node_host_memory_gb", value)

    @property
    @pulumi.getter(name="opendashboardNodeHostOcpuCount")
    def opendashboard_node_host_ocpu_count(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        """
        return pulumi.get(self, "opendashboard_node_host_ocpu_count")

    @opendashboard_node_host_ocpu_count.setter
    def opendashboard_node_host_ocpu_count(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "opendashboard_node_host_ocpu_count", value)

    @property
    @pulumi.getter(name="opendashboardPrivateIp")
    def opendashboard_private_ip(self) -> Optional[pulumi.Input[str]]:
        """
        The private IP address for the cluster's OpenSearch Dashboard.
        """
        return pulumi.get(self, "opendashboard_private_ip")

    @opendashboard_private_ip.setter
    def opendashboard_private_ip(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "opendashboard_private_ip", value)

    @property
    @pulumi.getter(name="opensearchFqdn")
    def opensearch_fqdn(self) -> Optional[pulumi.Input[str]]:
        """
        The fully qualified domain name (FQDN) for the cluster's API endpoint.
        """
        return pulumi.get(self, "opensearch_fqdn")

    @opensearch_fqdn.setter
    def opensearch_fqdn(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "opensearch_fqdn", value)

    @property
    @pulumi.getter(name="opensearchPrivateIp")
    def opensearch_private_ip(self) -> Optional[pulumi.Input[str]]:
        """
        The cluster's private IP address.
        """
        return pulumi.get(self, "opensearch_private_ip")

    @opensearch_private_ip.setter
    def opensearch_private_ip(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "opensearch_private_ip", value)

    @property
    @pulumi.getter(name="securityMasterUserName")
    def security_master_user_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The name of the master user that are used to manage security config
        """
        return pulumi.get(self, "security_master_user_name")

    @security_master_user_name.setter
    def security_master_user_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_master_user_name", value)

    @property
    @pulumi.getter(name="securityMasterUserPasswordHash")
    def security_master_user_password_hash(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The password hash of the master user that are used to manage security config
        """
        return pulumi.get(self, "security_master_user_password_hash")

    @security_master_user_password_hash.setter
    def security_master_user_password_hash(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_master_user_password_hash", value)

    @property
    @pulumi.getter(name="securityMode")
    def security_mode(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The security mode of the cluster.
        """
        return pulumi.get(self, "security_mode")

    @security_mode.setter
    def security_mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_mode", value)

    @property
    @pulumi.getter(name="softwareVersion")
    def software_version(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The version of the software the cluster is running.
        """
        return pulumi.get(self, "software_version")

    @software_version.setter
    def software_version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "software_version", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        """
        The current state of the cluster.
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)

    @property
    @pulumi.getter(name="subnetCompartmentId")
    def subnet_compartment_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID for the compartment where the cluster's subnet is located.
        """
        return pulumi.get(self, "subnet_compartment_id")

    @subnet_compartment_id.setter
    def subnet_compartment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "subnet_compartment_id", value)

    @property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID of the cluster's subnet.
        """
        return pulumi.get(self, "subnet_id")

    @subnet_id.setter
    def subnet_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "subnet_id", value)

    @property
    @pulumi.getter(name="systemTags")
    def system_tags(self) -> Optional[pulumi.Input[Mapping[str, Any]]]:
        """
        Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        """
        return pulumi.get(self, "system_tags")

    @system_tags.setter
    def system_tags(self, value: Optional[pulumi.Input[Mapping[str, Any]]]):
        pulumi.set(self, "system_tags", value)

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> Optional[pulumi.Input[str]]:
        """
        The amount of time in milliseconds since the cluster was created.
        """
        return pulumi.get(self, "time_created")

    @time_created.setter
    def time_created(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_created", value)

    @property
    @pulumi.getter(name="timeDeleted")
    def time_deleted(self) -> Optional[pulumi.Input[str]]:
        """
        The amount of time in milliseconds since the cluster was updated.
        """
        return pulumi.get(self, "time_deleted")

    @time_deleted.setter
    def time_deleted(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_deleted", value)

    @property
    @pulumi.getter(name="timeUpdated")
    def time_updated(self) -> Optional[pulumi.Input[str]]:
        """
        The amount of time in milliseconds since the cluster was updated.
        """
        return pulumi.get(self, "time_updated")

    @time_updated.setter
    def time_updated(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "time_updated", value)

    @property
    @pulumi.getter(name="totalStorageGb")
    def total_storage_gb(self) -> Optional[pulumi.Input[int]]:
        """
        The size in GB of the cluster's total storage.
        """
        return pulumi.get(self, "total_storage_gb")

    @total_storage_gb.setter
    def total_storage_gb(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "total_storage_gb", value)

    @property
    @pulumi.getter(name="vcnCompartmentId")
    def vcn_compartment_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID for the compartment where the cluster's VCN is located.
        """
        return pulumi.get(self, "vcn_compartment_id")

    @vcn_compartment_id.setter
    def vcn_compartment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vcn_compartment_id", value)

    @property
    @pulumi.getter(name="vcnId")
    def vcn_id(self) -> Optional[pulumi.Input[str]]:
        """
        The OCID of the cluster's VCN.


        ** IMPORTANT **
        Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        return pulumi.get(self, "vcn_id")

    @vcn_id.setter
    def vcn_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vcn_id", value)


class Cluster(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 data_node_count: Optional[pulumi.Input[int]] = None,
                 data_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 data_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 data_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 data_node_host_type: Optional[pulumi.Input[str]] = None,
                 data_node_storage_gb: Optional[pulumi.Input[int]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 master_node_count: Optional[pulumi.Input[int]] = None,
                 master_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 master_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 master_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 master_node_host_type: Optional[pulumi.Input[str]] = None,
                 opendashboard_node_count: Optional[pulumi.Input[int]] = None,
                 opendashboard_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 opendashboard_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 security_master_user_name: Optional[pulumi.Input[str]] = None,
                 security_master_user_password_hash: Optional[pulumi.Input[str]] = None,
                 security_mode: Optional[pulumi.Input[str]] = None,
                 software_version: Optional[pulumi.Input[str]] = None,
                 subnet_compartment_id: Optional[pulumi.Input[str]] = None,
                 subnet_id: Optional[pulumi.Input[str]] = None,
                 system_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 vcn_compartment_id: Optional[pulumi.Input[str]] = None,
                 vcn_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        This resource provides the Opensearch Cluster resource in Oracle Cloud Infrastructure Opensearch service.

        Creates a new OpensearchCluster.

        ## Prerequisites

        The below policies must be created in compartment before creating OpensearchCluster

        ##### {Compartment-Name} - Name of  your compartment
        ```python
        import pulumi
        ```

        For latest documentation on OpenSearch use please refer to https://docs.oracle.com/en-us/iaas/Content/search-opensearch/home.htm\\
        Required permissions: https://docs.oracle.com/en-us/iaas/Content/search-opensearch/Concepts/ocisearchpermissions.htm

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_opensearch_cluster = oci.opensearch.Cluster("testOpensearchCluster",
            compartment_id=var["compartment_id"],
            data_node_count=var["opensearch_cluster_data_node_count"],
            data_node_host_memory_gb=var["opensearch_cluster_data_node_host_memory_gb"],
            data_node_host_ocpu_count=var["opensearch_cluster_data_node_host_ocpu_count"],
            data_node_host_type=var["opensearch_cluster_data_node_host_type"],
            data_node_storage_gb=var["opensearch_cluster_data_node_storage_gb"],
            display_name=var["opensearch_cluster_display_name"],
            master_node_count=var["opensearch_cluster_master_node_count"],
            master_node_host_memory_gb=var["opensearch_cluster_master_node_host_memory_gb"],
            master_node_host_ocpu_count=var["opensearch_cluster_master_node_host_ocpu_count"],
            master_node_host_type=var["opensearch_cluster_master_node_host_type"],
            opendashboard_node_count=var["opensearch_cluster_opendashboard_node_count"],
            opendashboard_node_host_memory_gb=var["opensearch_cluster_opendashboard_node_host_memory_gb"],
            opendashboard_node_host_ocpu_count=var["opensearch_cluster_opendashboard_node_host_ocpu_count"],
            software_version=var["opensearch_cluster_software_version"],
            subnet_compartment_id=oci_identity_compartment["test_compartment"]["id"],
            subnet_id=oci_core_subnet["test_subnet"]["id"],
            vcn_compartment_id=oci_identity_compartment["test_compartment"]["id"],
            vcn_id=oci_core_vcn["test_vcn"]["id"],
            data_node_host_bare_metal_shape=var["opensearch_cluster_data_node_host_bare_metal_shape"],
            defined_tags={
                "foo-namespace.bar-key": "value",
            },
            freeform_tags={
                "bar-key": "value",
            },
            master_node_host_bare_metal_shape=var["opensearch_cluster_master_node_host_bare_metal_shape"],
            security_master_user_name=oci_identity_user["test_user"]["name"],
            security_master_user_password_hash=var["opensearch_cluster_security_master_user_password_hash"],
            security_mode=var["opensearch_cluster_security_mode"],
            system_tags=var["opensearch_cluster_system_tags"])
        ```

        ## Import

        OpensearchClusters can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:Opensearch/cluster:Cluster test_opensearch_cluster "id"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment to create the cluster in.
        :param pulumi.Input[int] data_node_count: (Updatable) The number of data nodes to configure for the cluster.
        :param pulumi.Input[str] data_node_host_bare_metal_shape: The bare metal shape for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        :param pulumi.Input[str] data_node_host_type: TThe instance type for the cluster's data nodes.
        :param pulumi.Input[int] data_node_storage_gb: (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] display_name: (Updatable) The name of the cluster. Avoid entering confidential information.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[int] master_node_count: (Updatable) The number of master nodes to configure for the cluster.
        :param pulumi.Input[str] master_node_host_bare_metal_shape: The bare metal shape for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        :param pulumi.Input[str] master_node_host_type: The instance type for the cluster's master nodes.
        :param pulumi.Input[int] opendashboard_node_count: (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        :param pulumi.Input[int] opendashboard_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[int] opendashboard_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[str] security_master_user_name: (Updatable) The name of the master user that are used to manage security config
        :param pulumi.Input[str] security_master_user_password_hash: (Updatable) The password hash of the master user that are used to manage security config
        :param pulumi.Input[str] security_mode: (Updatable) The security mode of the cluster.
        :param pulumi.Input[str] software_version: (Updatable) The version of the software the cluster is running.
        :param pulumi.Input[str] subnet_compartment_id: The OCID for the compartment where the cluster's subnet is located.
        :param pulumi.Input[str] subnet_id: The OCID of the cluster's subnet.
        :param pulumi.Input[Mapping[str, Any]] system_tags: Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        :param pulumi.Input[str] vcn_compartment_id: The OCID for the compartment where the cluster's VCN is located.
        :param pulumi.Input[str] vcn_id: The OCID of the cluster's VCN.
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ClusterArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource provides the Opensearch Cluster resource in Oracle Cloud Infrastructure Opensearch service.

        Creates a new OpensearchCluster.

        ## Prerequisites

        The below policies must be created in compartment before creating OpensearchCluster

        ##### {Compartment-Name} - Name of  your compartment
        ```python
        import pulumi
        ```

        For latest documentation on OpenSearch use please refer to https://docs.oracle.com/en-us/iaas/Content/search-opensearch/home.htm\\
        Required permissions: https://docs.oracle.com/en-us/iaas/Content/search-opensearch/Concepts/ocisearchpermissions.htm

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_opensearch_cluster = oci.opensearch.Cluster("testOpensearchCluster",
            compartment_id=var["compartment_id"],
            data_node_count=var["opensearch_cluster_data_node_count"],
            data_node_host_memory_gb=var["opensearch_cluster_data_node_host_memory_gb"],
            data_node_host_ocpu_count=var["opensearch_cluster_data_node_host_ocpu_count"],
            data_node_host_type=var["opensearch_cluster_data_node_host_type"],
            data_node_storage_gb=var["opensearch_cluster_data_node_storage_gb"],
            display_name=var["opensearch_cluster_display_name"],
            master_node_count=var["opensearch_cluster_master_node_count"],
            master_node_host_memory_gb=var["opensearch_cluster_master_node_host_memory_gb"],
            master_node_host_ocpu_count=var["opensearch_cluster_master_node_host_ocpu_count"],
            master_node_host_type=var["opensearch_cluster_master_node_host_type"],
            opendashboard_node_count=var["opensearch_cluster_opendashboard_node_count"],
            opendashboard_node_host_memory_gb=var["opensearch_cluster_opendashboard_node_host_memory_gb"],
            opendashboard_node_host_ocpu_count=var["opensearch_cluster_opendashboard_node_host_ocpu_count"],
            software_version=var["opensearch_cluster_software_version"],
            subnet_compartment_id=oci_identity_compartment["test_compartment"]["id"],
            subnet_id=oci_core_subnet["test_subnet"]["id"],
            vcn_compartment_id=oci_identity_compartment["test_compartment"]["id"],
            vcn_id=oci_core_vcn["test_vcn"]["id"],
            data_node_host_bare_metal_shape=var["opensearch_cluster_data_node_host_bare_metal_shape"],
            defined_tags={
                "foo-namespace.bar-key": "value",
            },
            freeform_tags={
                "bar-key": "value",
            },
            master_node_host_bare_metal_shape=var["opensearch_cluster_master_node_host_bare_metal_shape"],
            security_master_user_name=oci_identity_user["test_user"]["name"],
            security_master_user_password_hash=var["opensearch_cluster_security_master_user_password_hash"],
            security_mode=var["opensearch_cluster_security_mode"],
            system_tags=var["opensearch_cluster_system_tags"])
        ```

        ## Import

        OpensearchClusters can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:Opensearch/cluster:Cluster test_opensearch_cluster "id"
        ```

        :param str resource_name: The name of the resource.
        :param ClusterArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ClusterArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 compartment_id: Optional[pulumi.Input[str]] = None,
                 data_node_count: Optional[pulumi.Input[int]] = None,
                 data_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 data_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 data_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 data_node_host_type: Optional[pulumi.Input[str]] = None,
                 data_node_storage_gb: Optional[pulumi.Input[int]] = None,
                 defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 master_node_count: Optional[pulumi.Input[int]] = None,
                 master_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
                 master_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 master_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 master_node_host_type: Optional[pulumi.Input[str]] = None,
                 opendashboard_node_count: Optional[pulumi.Input[int]] = None,
                 opendashboard_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
                 opendashboard_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
                 security_master_user_name: Optional[pulumi.Input[str]] = None,
                 security_master_user_password_hash: Optional[pulumi.Input[str]] = None,
                 security_mode: Optional[pulumi.Input[str]] = None,
                 software_version: Optional[pulumi.Input[str]] = None,
                 subnet_compartment_id: Optional[pulumi.Input[str]] = None,
                 subnet_id: Optional[pulumi.Input[str]] = None,
                 system_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
                 vcn_compartment_id: Optional[pulumi.Input[str]] = None,
                 vcn_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ClusterArgs.__new__(ClusterArgs)

            if compartment_id is None and not opts.urn:
                raise TypeError("Missing required property 'compartment_id'")
            __props__.__dict__["compartment_id"] = compartment_id
            if data_node_count is None and not opts.urn:
                raise TypeError("Missing required property 'data_node_count'")
            __props__.__dict__["data_node_count"] = data_node_count
            __props__.__dict__["data_node_host_bare_metal_shape"] = data_node_host_bare_metal_shape
            if data_node_host_memory_gb is None and not opts.urn:
                raise TypeError("Missing required property 'data_node_host_memory_gb'")
            __props__.__dict__["data_node_host_memory_gb"] = data_node_host_memory_gb
            if data_node_host_ocpu_count is None and not opts.urn:
                raise TypeError("Missing required property 'data_node_host_ocpu_count'")
            __props__.__dict__["data_node_host_ocpu_count"] = data_node_host_ocpu_count
            if data_node_host_type is None and not opts.urn:
                raise TypeError("Missing required property 'data_node_host_type'")
            __props__.__dict__["data_node_host_type"] = data_node_host_type
            if data_node_storage_gb is None and not opts.urn:
                raise TypeError("Missing required property 'data_node_storage_gb'")
            __props__.__dict__["data_node_storage_gb"] = data_node_storage_gb
            __props__.__dict__["defined_tags"] = defined_tags
            if display_name is None and not opts.urn:
                raise TypeError("Missing required property 'display_name'")
            __props__.__dict__["display_name"] = display_name
            __props__.__dict__["freeform_tags"] = freeform_tags
            if master_node_count is None and not opts.urn:
                raise TypeError("Missing required property 'master_node_count'")
            __props__.__dict__["master_node_count"] = master_node_count
            __props__.__dict__["master_node_host_bare_metal_shape"] = master_node_host_bare_metal_shape
            if master_node_host_memory_gb is None and not opts.urn:
                raise TypeError("Missing required property 'master_node_host_memory_gb'")
            __props__.__dict__["master_node_host_memory_gb"] = master_node_host_memory_gb
            if master_node_host_ocpu_count is None and not opts.urn:
                raise TypeError("Missing required property 'master_node_host_ocpu_count'")
            __props__.__dict__["master_node_host_ocpu_count"] = master_node_host_ocpu_count
            if master_node_host_type is None and not opts.urn:
                raise TypeError("Missing required property 'master_node_host_type'")
            __props__.__dict__["master_node_host_type"] = master_node_host_type
            if opendashboard_node_count is None and not opts.urn:
                raise TypeError("Missing required property 'opendashboard_node_count'")
            __props__.__dict__["opendashboard_node_count"] = opendashboard_node_count
            if opendashboard_node_host_memory_gb is None and not opts.urn:
                raise TypeError("Missing required property 'opendashboard_node_host_memory_gb'")
            __props__.__dict__["opendashboard_node_host_memory_gb"] = opendashboard_node_host_memory_gb
            if opendashboard_node_host_ocpu_count is None and not opts.urn:
                raise TypeError("Missing required property 'opendashboard_node_host_ocpu_count'")
            __props__.__dict__["opendashboard_node_host_ocpu_count"] = opendashboard_node_host_ocpu_count
            __props__.__dict__["security_master_user_name"] = security_master_user_name
            __props__.__dict__["security_master_user_password_hash"] = None if security_master_user_password_hash is None else pulumi.Output.secret(security_master_user_password_hash)
            __props__.__dict__["security_mode"] = security_mode
            if software_version is None and not opts.urn:
                raise TypeError("Missing required property 'software_version'")
            __props__.__dict__["software_version"] = software_version
            if subnet_compartment_id is None and not opts.urn:
                raise TypeError("Missing required property 'subnet_compartment_id'")
            __props__.__dict__["subnet_compartment_id"] = subnet_compartment_id
            if subnet_id is None and not opts.urn:
                raise TypeError("Missing required property 'subnet_id'")
            __props__.__dict__["subnet_id"] = subnet_id
            __props__.__dict__["system_tags"] = system_tags
            if vcn_compartment_id is None and not opts.urn:
                raise TypeError("Missing required property 'vcn_compartment_id'")
            __props__.__dict__["vcn_compartment_id"] = vcn_compartment_id
            if vcn_id is None and not opts.urn:
                raise TypeError("Missing required property 'vcn_id'")
            __props__.__dict__["vcn_id"] = vcn_id
            __props__.__dict__["availability_domains"] = None
            __props__.__dict__["fqdn"] = None
            __props__.__dict__["lifecycle_details"] = None
            __props__.__dict__["opendashboard_fqdn"] = None
            __props__.__dict__["opendashboard_private_ip"] = None
            __props__.__dict__["opensearch_fqdn"] = None
            __props__.__dict__["opensearch_private_ip"] = None
            __props__.__dict__["state"] = None
            __props__.__dict__["time_created"] = None
            __props__.__dict__["time_deleted"] = None
            __props__.__dict__["time_updated"] = None
            __props__.__dict__["total_storage_gb"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["securityMasterUserPasswordHash"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Cluster, __self__).__init__(
            'oci:Opensearch/cluster:Cluster',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            availability_domains: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            compartment_id: Optional[pulumi.Input[str]] = None,
            data_node_count: Optional[pulumi.Input[int]] = None,
            data_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
            data_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
            data_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
            data_node_host_type: Optional[pulumi.Input[str]] = None,
            data_node_storage_gb: Optional[pulumi.Input[int]] = None,
            defined_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            fqdn: Optional[pulumi.Input[str]] = None,
            freeform_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            lifecycle_details: Optional[pulumi.Input[str]] = None,
            master_node_count: Optional[pulumi.Input[int]] = None,
            master_node_host_bare_metal_shape: Optional[pulumi.Input[str]] = None,
            master_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
            master_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
            master_node_host_type: Optional[pulumi.Input[str]] = None,
            opendashboard_fqdn: Optional[pulumi.Input[str]] = None,
            opendashboard_node_count: Optional[pulumi.Input[int]] = None,
            opendashboard_node_host_memory_gb: Optional[pulumi.Input[int]] = None,
            opendashboard_node_host_ocpu_count: Optional[pulumi.Input[int]] = None,
            opendashboard_private_ip: Optional[pulumi.Input[str]] = None,
            opensearch_fqdn: Optional[pulumi.Input[str]] = None,
            opensearch_private_ip: Optional[pulumi.Input[str]] = None,
            security_master_user_name: Optional[pulumi.Input[str]] = None,
            security_master_user_password_hash: Optional[pulumi.Input[str]] = None,
            security_mode: Optional[pulumi.Input[str]] = None,
            software_version: Optional[pulumi.Input[str]] = None,
            state: Optional[pulumi.Input[str]] = None,
            subnet_compartment_id: Optional[pulumi.Input[str]] = None,
            subnet_id: Optional[pulumi.Input[str]] = None,
            system_tags: Optional[pulumi.Input[Mapping[str, Any]]] = None,
            time_created: Optional[pulumi.Input[str]] = None,
            time_deleted: Optional[pulumi.Input[str]] = None,
            time_updated: Optional[pulumi.Input[str]] = None,
            total_storage_gb: Optional[pulumi.Input[int]] = None,
            vcn_compartment_id: Optional[pulumi.Input[str]] = None,
            vcn_id: Optional[pulumi.Input[str]] = None) -> 'Cluster':
        """
        Get an existing Cluster resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] availability_domains: The availability domains to distribute the cluser nodes across.
        :param pulumi.Input[str] compartment_id: The OCID of the compartment to create the cluster in.
        :param pulumi.Input[int] data_node_count: (Updatable) The number of data nodes to configure for the cluster.
        :param pulumi.Input[str] data_node_host_bare_metal_shape: The bare metal shape for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[int] data_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        :param pulumi.Input[str] data_node_host_type: TThe instance type for the cluster's data nodes.
        :param pulumi.Input[int] data_node_storage_gb: (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        :param pulumi.Input[Mapping[str, Any]] defined_tags: (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        :param pulumi.Input[str] display_name: (Updatable) The name of the cluster. Avoid entering confidential information.
        :param pulumi.Input[str] fqdn: The fully qualified domain name (FQDN) for the cluster's API endpoint.
        :param pulumi.Input[Mapping[str, Any]] freeform_tags: (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        :param pulumi.Input[str] lifecycle_details: Additional information about the current lifecycle state of the cluster.
        :param pulumi.Input[int] master_node_count: (Updatable) The number of master nodes to configure for the cluster.
        :param pulumi.Input[str] master_node_host_bare_metal_shape: The bare metal shape for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        :param pulumi.Input[int] master_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        :param pulumi.Input[str] master_node_host_type: The instance type for the cluster's master nodes.
        :param pulumi.Input[str] opendashboard_fqdn: The fully qualified domain name (FQDN) for the cluster's OpenSearch Dashboard API endpoint.
        :param pulumi.Input[int] opendashboard_node_count: (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        :param pulumi.Input[int] opendashboard_node_host_memory_gb: (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[int] opendashboard_node_host_ocpu_count: (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        :param pulumi.Input[str] opendashboard_private_ip: The private IP address for the cluster's OpenSearch Dashboard.
        :param pulumi.Input[str] opensearch_fqdn: The fully qualified domain name (FQDN) for the cluster's API endpoint.
        :param pulumi.Input[str] opensearch_private_ip: The cluster's private IP address.
        :param pulumi.Input[str] security_master_user_name: (Updatable) The name of the master user that are used to manage security config
        :param pulumi.Input[str] security_master_user_password_hash: (Updatable) The password hash of the master user that are used to manage security config
        :param pulumi.Input[str] security_mode: (Updatable) The security mode of the cluster.
        :param pulumi.Input[str] software_version: (Updatable) The version of the software the cluster is running.
        :param pulumi.Input[str] state: The current state of the cluster.
        :param pulumi.Input[str] subnet_compartment_id: The OCID for the compartment where the cluster's subnet is located.
        :param pulumi.Input[str] subnet_id: The OCID of the cluster's subnet.
        :param pulumi.Input[Mapping[str, Any]] system_tags: Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        :param pulumi.Input[str] time_created: The amount of time in milliseconds since the cluster was created.
        :param pulumi.Input[str] time_deleted: The amount of time in milliseconds since the cluster was updated.
        :param pulumi.Input[str] time_updated: The amount of time in milliseconds since the cluster was updated.
        :param pulumi.Input[int] total_storage_gb: The size in GB of the cluster's total storage.
        :param pulumi.Input[str] vcn_compartment_id: The OCID for the compartment where the cluster's VCN is located.
        :param pulumi.Input[str] vcn_id: The OCID of the cluster's VCN.
               
               
               ** IMPORTANT **
               Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ClusterState.__new__(_ClusterState)

        __props__.__dict__["availability_domains"] = availability_domains
        __props__.__dict__["compartment_id"] = compartment_id
        __props__.__dict__["data_node_count"] = data_node_count
        __props__.__dict__["data_node_host_bare_metal_shape"] = data_node_host_bare_metal_shape
        __props__.__dict__["data_node_host_memory_gb"] = data_node_host_memory_gb
        __props__.__dict__["data_node_host_ocpu_count"] = data_node_host_ocpu_count
        __props__.__dict__["data_node_host_type"] = data_node_host_type
        __props__.__dict__["data_node_storage_gb"] = data_node_storage_gb
        __props__.__dict__["defined_tags"] = defined_tags
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["fqdn"] = fqdn
        __props__.__dict__["freeform_tags"] = freeform_tags
        __props__.__dict__["lifecycle_details"] = lifecycle_details
        __props__.__dict__["master_node_count"] = master_node_count
        __props__.__dict__["master_node_host_bare_metal_shape"] = master_node_host_bare_metal_shape
        __props__.__dict__["master_node_host_memory_gb"] = master_node_host_memory_gb
        __props__.__dict__["master_node_host_ocpu_count"] = master_node_host_ocpu_count
        __props__.__dict__["master_node_host_type"] = master_node_host_type
        __props__.__dict__["opendashboard_fqdn"] = opendashboard_fqdn
        __props__.__dict__["opendashboard_node_count"] = opendashboard_node_count
        __props__.__dict__["opendashboard_node_host_memory_gb"] = opendashboard_node_host_memory_gb
        __props__.__dict__["opendashboard_node_host_ocpu_count"] = opendashboard_node_host_ocpu_count
        __props__.__dict__["opendashboard_private_ip"] = opendashboard_private_ip
        __props__.__dict__["opensearch_fqdn"] = opensearch_fqdn
        __props__.__dict__["opensearch_private_ip"] = opensearch_private_ip
        __props__.__dict__["security_master_user_name"] = security_master_user_name
        __props__.__dict__["security_master_user_password_hash"] = security_master_user_password_hash
        __props__.__dict__["security_mode"] = security_mode
        __props__.__dict__["software_version"] = software_version
        __props__.__dict__["state"] = state
        __props__.__dict__["subnet_compartment_id"] = subnet_compartment_id
        __props__.__dict__["subnet_id"] = subnet_id
        __props__.__dict__["system_tags"] = system_tags
        __props__.__dict__["time_created"] = time_created
        __props__.__dict__["time_deleted"] = time_deleted
        __props__.__dict__["time_updated"] = time_updated
        __props__.__dict__["total_storage_gb"] = total_storage_gb
        __props__.__dict__["vcn_compartment_id"] = vcn_compartment_id
        __props__.__dict__["vcn_id"] = vcn_id
        return Cluster(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="availabilityDomains")
    def availability_domains(self) -> pulumi.Output[Sequence[str]]:
        """
        The availability domains to distribute the cluser nodes across.
        """
        return pulumi.get(self, "availability_domains")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> pulumi.Output[str]:
        """
        The OCID of the compartment to create the cluster in.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="dataNodeCount")
    def data_node_count(self) -> pulumi.Output[int]:
        """
        (Updatable) The number of data nodes to configure for the cluster.
        """
        return pulumi.get(self, "data_node_count")

    @property
    @pulumi.getter(name="dataNodeHostBareMetalShape")
    def data_node_host_bare_metal_shape(self) -> pulumi.Output[str]:
        """
        The bare metal shape for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_bare_metal_shape")

    @property
    @pulumi.getter(name="dataNodeHostMemoryGb")
    def data_node_host_memory_gb(self) -> pulumi.Output[int]:
        """
        (Updatable) The amount of memory in GB, to configure per node for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_memory_gb")

    @property
    @pulumi.getter(name="dataNodeHostOcpuCount")
    def data_node_host_ocpu_count(self) -> pulumi.Output[int]:
        """
        (Updatable) The number of OCPUs to configure for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_ocpu_count")

    @property
    @pulumi.getter(name="dataNodeHostType")
    def data_node_host_type(self) -> pulumi.Output[str]:
        """
        TThe instance type for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_host_type")

    @property
    @pulumi.getter(name="dataNodeStorageGb")
    def data_node_storage_gb(self) -> pulumi.Output[int]:
        """
        (Updatable) The amount of storage in GB, to configure per node for the cluster's data nodes.
        """
        return pulumi.get(self, "data_node_storage_gb")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        (Updatable) Defined tags for this resource. Each key is predefined and scoped to a namespace. Example: `{"foo-namespace.bar-key": "value"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        (Updatable) The name of the cluster. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def fqdn(self) -> pulumi.Output[str]:
        """
        The fully qualified domain name (FQDN) for the cluster's API endpoint.
        """
        return pulumi.get(self, "fqdn")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        (Updatable) Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only. Example: `{"bar-key": "value"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> pulumi.Output[str]:
        """
        Additional information about the current lifecycle state of the cluster.
        """
        return pulumi.get(self, "lifecycle_details")

    @property
    @pulumi.getter(name="masterNodeCount")
    def master_node_count(self) -> pulumi.Output[int]:
        """
        (Updatable) The number of master nodes to configure for the cluster.
        """
        return pulumi.get(self, "master_node_count")

    @property
    @pulumi.getter(name="masterNodeHostBareMetalShape")
    def master_node_host_bare_metal_shape(self) -> pulumi.Output[str]:
        """
        The bare metal shape for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_bare_metal_shape")

    @property
    @pulumi.getter(name="masterNodeHostMemoryGb")
    def master_node_host_memory_gb(self) -> pulumi.Output[int]:
        """
        (Updatable) The amount of memory in GB, to configure per node for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_memory_gb")

    @property
    @pulumi.getter(name="masterNodeHostOcpuCount")
    def master_node_host_ocpu_count(self) -> pulumi.Output[int]:
        """
        (Updatable) The number of OCPUs to configure for the cluser's master nodes.
        """
        return pulumi.get(self, "master_node_host_ocpu_count")

    @property
    @pulumi.getter(name="masterNodeHostType")
    def master_node_host_type(self) -> pulumi.Output[str]:
        """
        The instance type for the cluster's master nodes.
        """
        return pulumi.get(self, "master_node_host_type")

    @property
    @pulumi.getter(name="opendashboardFqdn")
    def opendashboard_fqdn(self) -> pulumi.Output[str]:
        """
        The fully qualified domain name (FQDN) for the cluster's OpenSearch Dashboard API endpoint.
        """
        return pulumi.get(self, "opendashboard_fqdn")

    @property
    @pulumi.getter(name="opendashboardNodeCount")
    def opendashboard_node_count(self) -> pulumi.Output[int]:
        """
        (Updatable) The number of OpenSearch Dashboard nodes to configure for the cluster.
        """
        return pulumi.get(self, "opendashboard_node_count")

    @property
    @pulumi.getter(name="opendashboardNodeHostMemoryGb")
    def opendashboard_node_host_memory_gb(self) -> pulumi.Output[int]:
        """
        (Updatable) The amount of memory in GB, to configure for the cluster's OpenSearch Dashboard nodes.
        """
        return pulumi.get(self, "opendashboard_node_host_memory_gb")

    @property
    @pulumi.getter(name="opendashboardNodeHostOcpuCount")
    def opendashboard_node_host_ocpu_count(self) -> pulumi.Output[int]:
        """
        (Updatable) The number of OCPUs to configure for the cluster's OpenSearch Dashboard nodes.
        """
        return pulumi.get(self, "opendashboard_node_host_ocpu_count")

    @property
    @pulumi.getter(name="opendashboardPrivateIp")
    def opendashboard_private_ip(self) -> pulumi.Output[str]:
        """
        The private IP address for the cluster's OpenSearch Dashboard.
        """
        return pulumi.get(self, "opendashboard_private_ip")

    @property
    @pulumi.getter(name="opensearchFqdn")
    def opensearch_fqdn(self) -> pulumi.Output[str]:
        """
        The fully qualified domain name (FQDN) for the cluster's API endpoint.
        """
        return pulumi.get(self, "opensearch_fqdn")

    @property
    @pulumi.getter(name="opensearchPrivateIp")
    def opensearch_private_ip(self) -> pulumi.Output[str]:
        """
        The cluster's private IP address.
        """
        return pulumi.get(self, "opensearch_private_ip")

    @property
    @pulumi.getter(name="securityMasterUserName")
    def security_master_user_name(self) -> pulumi.Output[str]:
        """
        (Updatable) The name of the master user that are used to manage security config
        """
        return pulumi.get(self, "security_master_user_name")

    @property
    @pulumi.getter(name="securityMasterUserPasswordHash")
    def security_master_user_password_hash(self) -> pulumi.Output[str]:
        """
        (Updatable) The password hash of the master user that are used to manage security config
        """
        return pulumi.get(self, "security_master_user_password_hash")

    @property
    @pulumi.getter(name="securityMode")
    def security_mode(self) -> pulumi.Output[str]:
        """
        (Updatable) The security mode of the cluster.
        """
        return pulumi.get(self, "security_mode")

    @property
    @pulumi.getter(name="softwareVersion")
    def software_version(self) -> pulumi.Output[str]:
        """
        (Updatable) The version of the software the cluster is running.
        """
        return pulumi.get(self, "software_version")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        """
        The current state of the cluster.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="subnetCompartmentId")
    def subnet_compartment_id(self) -> pulumi.Output[str]:
        """
        The OCID for the compartment where the cluster's subnet is located.
        """
        return pulumi.get(self, "subnet_compartment_id")

    @property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> pulumi.Output[str]:
        """
        The OCID of the cluster's subnet.
        """
        return pulumi.get(self, "subnet_id")

    @property
    @pulumi.getter(name="systemTags")
    def system_tags(self) -> pulumi.Output[Mapping[str, Any]]:
        """
        Usage of system tag keys. These predefined keys are scoped to namespaces. Example: `{"orcl-cloud.free-tier-retained": "true"}`
        """
        return pulumi.get(self, "system_tags")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> pulumi.Output[str]:
        """
        The amount of time in milliseconds since the cluster was created.
        """
        return pulumi.get(self, "time_created")

    @property
    @pulumi.getter(name="timeDeleted")
    def time_deleted(self) -> pulumi.Output[str]:
        """
        The amount of time in milliseconds since the cluster was updated.
        """
        return pulumi.get(self, "time_deleted")

    @property
    @pulumi.getter(name="timeUpdated")
    def time_updated(self) -> pulumi.Output[str]:
        """
        The amount of time in milliseconds since the cluster was updated.
        """
        return pulumi.get(self, "time_updated")

    @property
    @pulumi.getter(name="totalStorageGb")
    def total_storage_gb(self) -> pulumi.Output[int]:
        """
        The size in GB of the cluster's total storage.
        """
        return pulumi.get(self, "total_storage_gb")

    @property
    @pulumi.getter(name="vcnCompartmentId")
    def vcn_compartment_id(self) -> pulumi.Output[str]:
        """
        The OCID for the compartment where the cluster's VCN is located.
        """
        return pulumi.get(self, "vcn_compartment_id")

    @property
    @pulumi.getter(name="vcnId")
    def vcn_id(self) -> pulumi.Output[str]:
        """
        The OCID of the cluster's VCN.


        ** IMPORTANT **
        Any change to a property that does not support update will force the destruction and recreation of the resource with the new property values
        """
        return pulumi.get(self, "vcn_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetResolverEndpointResult',
    'AwaitableGetResolverEndpointResult',
    'get_resolver_endpoint',
    'get_resolver_endpoint_output',
]

@pulumi.output_type
class GetResolverEndpointResult:
    """
    A collection of values returned by getResolverEndpoint.
    """
    def __init__(__self__, compartment_id=None, endpoint_type=None, forwarding_address=None, id=None, is_forwarding=None, is_listening=None, listening_address=None, name=None, nsg_ids=None, resolver_endpoint_name=None, resolver_id=None, scope=None, self=None, state=None, subnet_id=None, time_created=None, time_updated=None):
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if endpoint_type and not isinstance(endpoint_type, str):
            raise TypeError("Expected argument 'endpoint_type' to be a str")
        pulumi.set(__self__, "endpoint_type", endpoint_type)
        if forwarding_address and not isinstance(forwarding_address, str):
            raise TypeError("Expected argument 'forwarding_address' to be a str")
        pulumi.set(__self__, "forwarding_address", forwarding_address)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if is_forwarding and not isinstance(is_forwarding, bool):
            raise TypeError("Expected argument 'is_forwarding' to be a bool")
        pulumi.set(__self__, "is_forwarding", is_forwarding)
        if is_listening and not isinstance(is_listening, bool):
            raise TypeError("Expected argument 'is_listening' to be a bool")
        pulumi.set(__self__, "is_listening", is_listening)
        if listening_address and not isinstance(listening_address, str):
            raise TypeError("Expected argument 'listening_address' to be a str")
        pulumi.set(__self__, "listening_address", listening_address)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if nsg_ids and not isinstance(nsg_ids, list):
            raise TypeError("Expected argument 'nsg_ids' to be a list")
        pulumi.set(__self__, "nsg_ids", nsg_ids)
        if resolver_endpoint_name and not isinstance(resolver_endpoint_name, str):
            raise TypeError("Expected argument 'resolver_endpoint_name' to be a str")
        pulumi.set(__self__, "resolver_endpoint_name", resolver_endpoint_name)
        if resolver_id and not isinstance(resolver_id, str):
            raise TypeError("Expected argument 'resolver_id' to be a str")
        pulumi.set(__self__, "resolver_id", resolver_id)
        if scope and not isinstance(scope, str):
            raise TypeError("Expected argument 'scope' to be a str")
        pulumi.set(__self__, "scope", scope)
        if self and not isinstance(self, str):
            raise TypeError("Expected argument 'self' to be a str")
        pulumi.set(__self__, "self", self)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if subnet_id and not isinstance(subnet_id, str):
            raise TypeError("Expected argument 'subnet_id' to be a str")
        pulumi.set(__self__, "subnet_id", subnet_id)
        if time_created and not isinstance(time_created, str):
            raise TypeError("Expected argument 'time_created' to be a str")
        pulumi.set(__self__, "time_created", time_created)
        if time_updated and not isinstance(time_updated, str):
            raise TypeError("Expected argument 'time_updated' to be a str")
        pulumi.set(__self__, "time_updated", time_updated)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The OCID of the owning compartment. This will match the resolver that the resolver endpoint is under and will be updated if the resolver's compartment is changed.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="endpointType")
    def endpoint_type(self) -> str:
        """
        The type of resolver endpoint. VNIC is currently the only supported type.
        """
        return pulumi.get(self, "endpoint_type")

    @property
    @pulumi.getter(name="forwardingAddress")
    def forwarding_address(self) -> str:
        """
        An IP address from which forwarded queries may be sent. For VNIC endpoints, this IP address must be part of the subnet and will be assigned by the system if unspecified when isForwarding is true.
        """
        return pulumi.get(self, "forwarding_address")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="isForwarding")
    def is_forwarding(self) -> bool:
        """
        A Boolean flag indicating whether or not the resolver endpoint is for forwarding.
        """
        return pulumi.get(self, "is_forwarding")

    @property
    @pulumi.getter(name="isListening")
    def is_listening(self) -> bool:
        """
        A Boolean flag indicating whether or not the resolver endpoint is for listening.
        """
        return pulumi.get(self, "is_listening")

    @property
    @pulumi.getter(name="listeningAddress")
    def listening_address(self) -> str:
        """
        An IP address to listen to queries on. For VNIC endpoints this IP address must be part of the subnet and will be assigned by the system if unspecified when isListening is true.
        """
        return pulumi.get(self, "listening_address")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the resolver endpoint. Must be unique, case-insensitive, within the resolver.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="nsgIds")
    def nsg_ids(self) -> Sequence[str]:
        """
        An array of network security group OCIDs for the resolver endpoint. These must be part of the VCN that the resolver endpoint is a part of.
        """
        return pulumi.get(self, "nsg_ids")

    @property
    @pulumi.getter(name="resolverEndpointName")
    def resolver_endpoint_name(self) -> str:
        return pulumi.get(self, "resolver_endpoint_name")

    @property
    @pulumi.getter(name="resolverId")
    def resolver_id(self) -> str:
        return pulumi.get(self, "resolver_id")

    @property
    @pulumi.getter
    def scope(self) -> Optional[str]:
        return pulumi.get(self, "scope")

    @property
    @pulumi.getter
    def self(self) -> str:
        """
        The canonical absolute URL of the resource.
        """
        return pulumi.get(self, "self")

    @property
    @pulumi.getter
    def state(self) -> str:
        """
        The current state of the resource.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> str:
        """
        The OCID of a subnet. Must be part of the VCN that the resolver is attached to.
        """
        return pulumi.get(self, "subnet_id")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> str:
        """
        The date and time the resource was created in "YYYY-MM-ddThh:mm:ssZ" format with a Z offset, as defined by RFC 3339.
        """
        return pulumi.get(self, "time_created")

    @property
    @pulumi.getter(name="timeUpdated")
    def time_updated(self) -> str:
        """
        The date and time the resource was last updated in "YYYY-MM-ddThh:mm:ssZ" format with a Z offset, as defined by RFC 3339.
        """
        return pulumi.get(self, "time_updated")


class AwaitableGetResolverEndpointResult(GetResolverEndpointResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetResolverEndpointResult(
            compartment_id=self.compartment_id,
            endpoint_type=self.endpoint_type,
            forwarding_address=self.forwarding_address,
            id=self.id,
            is_forwarding=self.is_forwarding,
            is_listening=self.is_listening,
            listening_address=self.listening_address,
            name=self.name,
            nsg_ids=self.nsg_ids,
            resolver_endpoint_name=self.resolver_endpoint_name,
            resolver_id=self.resolver_id,
            scope=self.scope,
            self=self.self,
            state=self.state,
            subnet_id=self.subnet_id,
            time_created=self.time_created,
            time_updated=self.time_updated)


def get_resolver_endpoint(resolver_endpoint_name: Optional[str] = None,
                          resolver_id: Optional[str] = None,
                          scope: Optional[str] = None,
                          opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetResolverEndpointResult:
    """
    This data source provides details about a specific Resolver Endpoint resource in Oracle Cloud Infrastructure DNS service.

    Gets information about a specific resolver endpoint. Note that attempting to get a resolver endpoint
    in the DELETED lifecycle state will result in a `404` response to be consistent with other operations of the
    API. Requires a `PRIVATE` scope query parameter.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_resolver_endpoint = oci.Dns.get_resolver_endpoint(resolver_endpoint_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        resolver_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        scope="PRIVATE")
    ```


    :param str resolver_endpoint_name: The name of the target resolver endpoint.
    :param str resolver_id: The OCID of the target resolver.
    :param str scope: Value must be `PRIVATE` when listing private name resolver endpoints.
    """
    __args__ = dict()
    __args__['resolverEndpointName'] = resolver_endpoint_name
    __args__['resolverId'] = resolver_id
    __args__['scope'] = scope
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Dns/getResolverEndpoint:getResolverEndpoint', __args__, opts=opts, typ=GetResolverEndpointResult).value

    return AwaitableGetResolverEndpointResult(
        compartment_id=__ret__.compartment_id,
        endpoint_type=__ret__.endpoint_type,
        forwarding_address=__ret__.forwarding_address,
        id=__ret__.id,
        is_forwarding=__ret__.is_forwarding,
        is_listening=__ret__.is_listening,
        listening_address=__ret__.listening_address,
        name=__ret__.name,
        nsg_ids=__ret__.nsg_ids,
        resolver_endpoint_name=__ret__.resolver_endpoint_name,
        resolver_id=__ret__.resolver_id,
        scope=__ret__.scope,
        self=__ret__.self,
        state=__ret__.state,
        subnet_id=__ret__.subnet_id,
        time_created=__ret__.time_created,
        time_updated=__ret__.time_updated)


@_utilities.lift_output_func(get_resolver_endpoint)
def get_resolver_endpoint_output(resolver_endpoint_name: Optional[pulumi.Input[str]] = None,
                                 resolver_id: Optional[pulumi.Input[str]] = None,
                                 scope: Optional[pulumi.Input[Optional[str]]] = None,
                                 opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetResolverEndpointResult]:
    """
    This data source provides details about a specific Resolver Endpoint resource in Oracle Cloud Infrastructure DNS service.

    Gets information about a specific resolver endpoint. Note that attempting to get a resolver endpoint
    in the DELETED lifecycle state will result in a `404` response to be consistent with other operations of the
    API. Requires a `PRIVATE` scope query parameter.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_resolver_endpoint = oci.Dns.get_resolver_endpoint(resolver_endpoint_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        resolver_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        scope="PRIVATE")
    ```


    :param str resolver_endpoint_name: The name of the target resolver endpoint.
    :param str resolver_id: The OCID of the target resolver.
    :param str scope: Value must be `PRIVATE` when listing private name resolver endpoints.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetContainerConfigurationResult',
    'AwaitableGetContainerConfigurationResult',
    'get_container_configuration',
    'get_container_configuration_output',
]

@pulumi.output_type
class GetContainerConfigurationResult:
    """
    A collection of values returned by getContainerConfiguration.
    """
    def __init__(__self__, compartment_id=None, id=None, is_repository_created_on_first_push=None, namespace=None):
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if is_repository_created_on_first_push and not isinstance(is_repository_created_on_first_push, bool):
            raise TypeError("Expected argument 'is_repository_created_on_first_push' to be a bool")
        pulumi.set(__self__, "is_repository_created_on_first_push", is_repository_created_on_first_push)
        if namespace and not isinstance(namespace, str):
            raise TypeError("Expected argument 'namespace' to be a str")
        pulumi.set(__self__, "namespace", namespace)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="isRepositoryCreatedOnFirstPush")
    def is_repository_created_on_first_push(self) -> bool:
        """
        Whether to create a new container repository when a container is pushed to a new repository path. Repositories created in this way belong to the root compartment.
        """
        return pulumi.get(self, "is_repository_created_on_first_push")

    @property
    @pulumi.getter
    def namespace(self) -> str:
        """
        The tenancy namespace used in the container repository path.
        """
        return pulumi.get(self, "namespace")


class AwaitableGetContainerConfigurationResult(GetContainerConfigurationResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetContainerConfigurationResult(
            compartment_id=self.compartment_id,
            id=self.id,
            is_repository_created_on_first_push=self.is_repository_created_on_first_push,
            namespace=self.namespace)


def get_container_configuration(compartment_id: Optional[str] = None,
                                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetContainerConfigurationResult:
    """
    This data source provides details about a specific Container Configuration resource in Oracle Cloud Infrastructure Artifacts service.

    Get container configuration.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_container_configuration = oci.Artifacts.get_container_configuration(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
    """
    __args__ = dict()
    __args__['compartmentId'] = compartment_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Artifacts/getContainerConfiguration:getContainerConfiguration', __args__, opts=opts, typ=GetContainerConfigurationResult).value

    return AwaitableGetContainerConfigurationResult(
        compartment_id=__ret__.compartment_id,
        id=__ret__.id,
        is_repository_created_on_first_push=__ret__.is_repository_created_on_first_push,
        namespace=__ret__.namespace)


@_utilities.lift_output_func(get_container_configuration)
def get_container_configuration_output(compartment_id: Optional[pulumi.Input[str]] = None,
                                       opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetContainerConfigurationResult]:
    """
    This data source provides details about a specific Container Configuration resource in Oracle Cloud Infrastructure Artifacts service.

    Get container configuration.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_container_configuration = oci.Artifacts.get_container_configuration(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetSddcsResult',
    'AwaitableGetSddcsResult',
    'get_sddcs',
    'get_sddcs_output',
]

@pulumi.output_type
class GetSddcsResult:
    """
    A collection of values returned by getSddcs.
    """
    def __init__(__self__, compartment_id=None, compute_availability_domain=None, display_name=None, filters=None, id=None, sddc_collections=None, state=None):
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if compute_availability_domain and not isinstance(compute_availability_domain, str):
            raise TypeError("Expected argument 'compute_availability_domain' to be a str")
        pulumi.set(__self__, "compute_availability_domain", compute_availability_domain)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if sddc_collections and not isinstance(sddc_collections, list):
            raise TypeError("Expected argument 'sddc_collections' to be a list")
        pulumi.set(__self__, "sddc_collections", sddc_collections)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment that contains the SDDC.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="computeAvailabilityDomain")
    def compute_availability_domain(self) -> Optional[str]:
        """
        The availability domain the ESXi hosts are running in. For Multi-AD SDDC, it is `multi-AD`.  Example: `Uocm:PHX-AD-1`, `multi-AD`
        """
        return pulumi.get(self, "compute_availability_domain")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[str]:
        """
        A descriptive name for the SDDC. It must be unique, start with a letter, and contain only letters, digits, whitespaces, dashes and underscores. Avoid entering confidential information.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetSddcsFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="sddcCollections")
    def sddc_collections(self) -> Sequence['outputs.GetSddcsSddcCollectionResult']:
        """
        The list of sddc_collection.
        """
        return pulumi.get(self, "sddc_collections")

    @property
    @pulumi.getter
    def state(self) -> Optional[str]:
        """
        The current state of the SDDC.
        """
        return pulumi.get(self, "state")


class AwaitableGetSddcsResult(GetSddcsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetSddcsResult(
            compartment_id=self.compartment_id,
            compute_availability_domain=self.compute_availability_domain,
            display_name=self.display_name,
            filters=self.filters,
            id=self.id,
            sddc_collections=self.sddc_collections,
            state=self.state)


def get_sddcs(compartment_id: Optional[str] = None,
              compute_availability_domain: Optional[str] = None,
              display_name: Optional[str] = None,
              filters: Optional[Sequence[pulumi.InputType['GetSddcsFilterArgs']]] = None,
              state: Optional[str] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetSddcsResult:
    """
    This data source provides the list of Sddcs in Oracle Cloud Infrastructure Oracle Cloud VMware Solution service.

    Lists the SDDCs in the specified compartment. The list can be
    filtered by display name or availability domain.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_sddcs = oci.Ocvp.get_sddcs(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        compute_availability_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        state=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
    :param str compute_availability_domain: The name of the availability domain that the Compute instances are running in.  Example: `Uocm:PHX-AD-1`
    :param str display_name: A filter to return only resources that match the given display name exactly.
    :param str state: The lifecycle state of the resource.
    """
    __args__ = dict()
    __args__['compartmentId'] = compartment_id
    __args__['computeAvailabilityDomain'] = compute_availability_domain
    __args__['displayName'] = display_name
    __args__['filters'] = filters
    __args__['state'] = state
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Ocvp/getSddcs:getSddcs', __args__, opts=opts, typ=GetSddcsResult).value

    return AwaitableGetSddcsResult(
        compartment_id=__ret__.compartment_id,
        compute_availability_domain=__ret__.compute_availability_domain,
        display_name=__ret__.display_name,
        filters=__ret__.filters,
        id=__ret__.id,
        sddc_collections=__ret__.sddc_collections,
        state=__ret__.state)


@_utilities.lift_output_func(get_sddcs)
def get_sddcs_output(compartment_id: Optional[pulumi.Input[str]] = None,
                     compute_availability_domain: Optional[pulumi.Input[Optional[str]]] = None,
                     display_name: Optional[pulumi.Input[Optional[str]]] = None,
                     filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetSddcsFilterArgs']]]]] = None,
                     state: Optional[pulumi.Input[Optional[str]]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetSddcsResult]:
    """
    This data source provides the list of Sddcs in Oracle Cloud Infrastructure Oracle Cloud VMware Solution service.

    Lists the SDDCs in the specified compartment. The list can be
    filtered by display name or availability domain.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_sddcs = oci.Ocvp.get_sddcs(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        compute_availability_domain=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        state=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
    :param str compute_availability_domain: The name of the availability domain that the Compute instances are running in.  Example: `Uocm:PHX-AD-1`
    :param str display_name: A filter to return only resources that match the given display name exactly.
    :param str state: The lifecycle state of the resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetTrailSequenceResult',
    'AwaitableGetTrailSequenceResult',
    'get_trail_sequence',
    'get_trail_sequence_output',
]

@pulumi.output_type
class GetTrailSequenceResult:
    """
    A collection of values returned by getTrailSequence.
    """
    def __init__(__self__, deployment_id=None, display_name=None, id=None, items=None, time_last_fetched=None, trail_file_id=None, trail_sequence_id=None):
        if deployment_id and not isinstance(deployment_id, str):
            raise TypeError("Expected argument 'deployment_id' to be a str")
        pulumi.set(__self__, "deployment_id", deployment_id)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if items and not isinstance(items, list):
            raise TypeError("Expected argument 'items' to be a list")
        pulumi.set(__self__, "items", items)
        if time_last_fetched and not isinstance(time_last_fetched, str):
            raise TypeError("Expected argument 'time_last_fetched' to be a str")
        pulumi.set(__self__, "time_last_fetched", time_last_fetched)
        if trail_file_id and not isinstance(trail_file_id, str):
            raise TypeError("Expected argument 'trail_file_id' to be a str")
        pulumi.set(__self__, "trail_file_id", trail_file_id)
        if trail_sequence_id and not isinstance(trail_sequence_id, str):
            raise TypeError("Expected argument 'trail_sequence_id' to be a str")
        pulumi.set(__self__, "trail_sequence_id", trail_sequence_id)

    @property
    @pulumi.getter(name="deploymentId")
    def deployment_id(self) -> str:
        return pulumi.get(self, "deployment_id")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        An object's Display Name.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def items(self) -> Sequence['outputs.GetTrailSequenceItemResult']:
        """
        An array of TrailSequences.
        """
        return pulumi.get(self, "items")

    @property
    @pulumi.getter(name="timeLastFetched")
    def time_last_fetched(self) -> str:
        """
        The time the data was last fetched from the deployment. The format is defined by [RFC3339](https://tools.ietf.org/html/rfc3339), such as `2016-08-25T21:10:29.600Z`.
        """
        return pulumi.get(self, "time_last_fetched")

    @property
    @pulumi.getter(name="trailFileId")
    def trail_file_id(self) -> str:
        return pulumi.get(self, "trail_file_id")

    @property
    @pulumi.getter(name="trailSequenceId")
    def trail_sequence_id(self) -> str:
        return pulumi.get(self, "trail_sequence_id")


class AwaitableGetTrailSequenceResult(GetTrailSequenceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetTrailSequenceResult(
            deployment_id=self.deployment_id,
            display_name=self.display_name,
            id=self.id,
            items=self.items,
            time_last_fetched=self.time_last_fetched,
            trail_file_id=self.trail_file_id,
            trail_sequence_id=self.trail_sequence_id)


def get_trail_sequence(deployment_id: Optional[str] = None,
                       display_name: Optional[str] = None,
                       trail_file_id: Optional[str] = None,
                       trail_sequence_id: Optional[str] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetTrailSequenceResult:
    """
    This data source provides details about a specific Trail Sequence resource in Oracle Cloud Infrastructure Golden Gate service.

    Lists the Trail Sequences for a TrailFile in a given deployment.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_trail_sequence = oci.GoldenGate.get_trail_sequence(deployment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        trail_file_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        trail_sequence_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str deployment_id: A unique Deployment identifier.
    :param str display_name: A filter to return only the resources that match the entire 'displayName' given.
    :param str trail_file_id: A Trail File identifier
    :param str trail_sequence_id: A Trail Sequence identifier
    """
    __args__ = dict()
    __args__['deploymentId'] = deployment_id
    __args__['displayName'] = display_name
    __args__['trailFileId'] = trail_file_id
    __args__['trailSequenceId'] = trail_sequence_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:GoldenGate/getTrailSequence:getTrailSequence', __args__, opts=opts, typ=GetTrailSequenceResult).value

    return AwaitableGetTrailSequenceResult(
        deployment_id=__ret__.deployment_id,
        display_name=__ret__.display_name,
        id=__ret__.id,
        items=__ret__.items,
        time_last_fetched=__ret__.time_last_fetched,
        trail_file_id=__ret__.trail_file_id,
        trail_sequence_id=__ret__.trail_sequence_id)


@_utilities.lift_output_func(get_trail_sequence)
def get_trail_sequence_output(deployment_id: Optional[pulumi.Input[str]] = None,
                              display_name: Optional[pulumi.Input[str]] = None,
                              trail_file_id: Optional[pulumi.Input[str]] = None,
                              trail_sequence_id: Optional[pulumi.Input[str]] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetTrailSequenceResult]:
    """
    This data source provides details about a specific Trail Sequence resource in Oracle Cloud Infrastructure Golden Gate service.

    Lists the Trail Sequences for a TrailFile in a given deployment.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_trail_sequence = oci.GoldenGate.get_trail_sequence(deployment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        trail_file_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        display_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        trail_sequence_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str deployment_id: A unique Deployment identifier.
    :param str display_name: A filter to return only the resources that match the entire 'displayName' given.
    :param str trail_file_id: A Trail File identifier
    :param str trail_sequence_id: A Trail Sequence identifier
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetRuleResult',
    'AwaitableGetRuleResult',
    'get_rule',
    'get_rule_output',
]

@pulumi.output_type
class GetRuleResult:
    """
    A collection of values returned by getRule.
    """
    def __init__(__self__, actions=None, compartment_id=None, condition=None, defined_tags=None, description=None, display_name=None, freeform_tags=None, id=None, is_enabled=None, lifecycle_message=None, rule_id=None, state=None, time_created=None):
        if actions and not isinstance(actions, list):
            raise TypeError("Expected argument 'actions' to be a list")
        pulumi.set(__self__, "actions", actions)
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if condition and not isinstance(condition, str):
            raise TypeError("Expected argument 'condition' to be a str")
        pulumi.set(__self__, "condition", condition)
        if defined_tags and not isinstance(defined_tags, dict):
            raise TypeError("Expected argument 'defined_tags' to be a dict")
        pulumi.set(__self__, "defined_tags", defined_tags)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if freeform_tags and not isinstance(freeform_tags, dict):
            raise TypeError("Expected argument 'freeform_tags' to be a dict")
        pulumi.set(__self__, "freeform_tags", freeform_tags)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if is_enabled and not isinstance(is_enabled, bool):
            raise TypeError("Expected argument 'is_enabled' to be a bool")
        pulumi.set(__self__, "is_enabled", is_enabled)
        if lifecycle_message and not isinstance(lifecycle_message, str):
            raise TypeError("Expected argument 'lifecycle_message' to be a str")
        pulumi.set(__self__, "lifecycle_message", lifecycle_message)
        if rule_id and not isinstance(rule_id, str):
            raise TypeError("Expected argument 'rule_id' to be a str")
        pulumi.set(__self__, "rule_id", rule_id)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if time_created and not isinstance(time_created, str):
            raise TypeError("Expected argument 'time_created' to be a str")
        pulumi.set(__self__, "time_created", time_created)

    @property
    @pulumi.getter
    def actions(self) -> Sequence['outputs.GetRuleActionResult']:
        """
        A list of one or more Action objects.
        """
        return pulumi.get(self, "actions")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment to which this rule belongs.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter
    def condition(self) -> str:
        """
        A filter that specifies the event that will trigger actions associated with this rule. A few  important things to remember about filters:
        * Fields not mentioned in the condition are ignored. You can create a valid filter that matches all events with two curly brackets: `{}`
        """
        return pulumi.get(self, "condition")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Mapping[str, Any]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        A string that describes the details of the rule. It does not have to be unique, and you can change it. Avoid entering confidential information.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        A string that describes the rule. It does not have to be unique, and you can change it. Avoid entering confidential information.  Example: `"This rule sends a notification upon completion of DbaaS backup."`
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Mapping[str, Any]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. Exists for cross-compatibility only. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of this rule.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="isEnabled")
    def is_enabled(self) -> bool:
        """
        Whether or not this rule is currently enabled.  Example: `true`
        """
        return pulumi.get(self, "is_enabled")

    @property
    @pulumi.getter(name="lifecycleMessage")
    def lifecycle_message(self) -> str:
        """
        A message generated by the Events service about the current state of this rule.
        """
        return pulumi.get(self, "lifecycle_message")

    @property
    @pulumi.getter(name="ruleId")
    def rule_id(self) -> str:
        return pulumi.get(self, "rule_id")

    @property
    @pulumi.getter
    def state(self) -> str:
        """
        The current state of the rule.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> str:
        """
        The time this rule was created, expressed in [RFC 3339](https://tools.ietf.org/html/rfc3339) timestamp format.  Example: `2018-09-12T22:47:12.613Z`
        """
        return pulumi.get(self, "time_created")


class AwaitableGetRuleResult(GetRuleResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRuleResult(
            actions=self.actions,
            compartment_id=self.compartment_id,
            condition=self.condition,
            defined_tags=self.defined_tags,
            description=self.description,
            display_name=self.display_name,
            freeform_tags=self.freeform_tags,
            id=self.id,
            is_enabled=self.is_enabled,
            lifecycle_message=self.lifecycle_message,
            rule_id=self.rule_id,
            state=self.state,
            time_created=self.time_created)


def get_rule(rule_id: Optional[str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetRuleResult:
    """
    This data source provides details about a specific Rule resource in Oracle Cloud Infrastructure Events service.

    Retrieves a rule.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_rule = oci.Events.get_rule(rule_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str rule_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of this rule.
    """
    __args__ = dict()
    __args__['ruleId'] = rule_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Events/getRule:getRule', __args__, opts=opts, typ=GetRuleResult).value

    return AwaitableGetRuleResult(
        actions=__ret__.actions,
        compartment_id=__ret__.compartment_id,
        condition=__ret__.condition,
        defined_tags=__ret__.defined_tags,
        description=__ret__.description,
        display_name=__ret__.display_name,
        freeform_tags=__ret__.freeform_tags,
        id=__ret__.id,
        is_enabled=__ret__.is_enabled,
        lifecycle_message=__ret__.lifecycle_message,
        rule_id=__ret__.rule_id,
        state=__ret__.state,
        time_created=__ret__.time_created)


@_utilities.lift_output_func(get_rule)
def get_rule_output(rule_id: Optional[pulumi.Input[str]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetRuleResult]:
    """
    This data source provides details about a specific Rule resource in Oracle Cloud Infrastructure Events service.

    Retrieves a rule.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_rule = oci.Events.get_rule(rule_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str rule_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of this rule.
    """
    ...
